import logging

import click
from datahub.ingestion.graph.config import ClientMode
from datahub.sdk.main_client import DataHubClient
from datahub.telemetry import telemetry
from fastmcp.server.middleware.logging import LoggingMiddleware
from typing_extensions import Literal

from mcp_server_datahub._telemetry import TelemetryMiddleware
from mcp_server_datahub._version import __version__
from mcp_server_datahub.mcp_server import mcp, with_datahub_client

logging.basicConfig(level=logging.INFO)


@click.command()
@click.version_option(version=__version__)
@click.option(
    "--transport",
    type=click.Choice(["stdio", "sse", "http"]),
    default="stdio",
)
@click.option(
    "--debug",
    is_flag=True,
    default=False,
)
@telemetry.with_telemetry(
    capture_kwargs=["transport"],
)
def main(transport: Literal["stdio", "sse", "http"], debug: bool) -> None:
    client = DataHubClient.from_env(
        client_mode=ClientMode.SDK,
        datahub_component=f"mcp-server-datahub/{__version__}",
    )

    if debug:
        # logging.getLogger("datahub").setLevel(logging.DEBUG)
        mcp.add_middleware(LoggingMiddleware(include_payloads=True))
    mcp.add_middleware(TelemetryMiddleware())

    with with_datahub_client(client):
        if transport == "http":
            mcp.run(transport=transport, show_banner=False, stateless_http=True)
        else:
            mcp.run(transport=transport, show_banner=False)


if __name__ == "__main__":
    main()



from typing import Any

import mcp.types as mt
from datahub.telemetry import telemetry
from datahub.utilities.perf_timer import PerfTimer
from fastmcp.server.middleware import CallNext, Middleware, MiddlewareContext

from mcp_server_datahub._version import __version__

telemetry.telemetry_instance.add_global_property(
    "mcp_server_datahub_version", __version__
)


class TelemetryMiddleware(Middleware):
    """Middleware that logs tool calls."""

    async def on_call_tool(
        self,
        context: MiddlewareContext[mt.CallToolRequestParams],
        call_next: CallNext[mt.CallToolRequestParams, mt.CallToolResult],
    ) -> mt.CallToolResult:
        telemetry_data: dict[str, Any] = {}
        with PerfTimer() as timer:
            telemetry_data = {
                "tool": context.message.name,
                "source": context.source,
                "type": context.type,
                "method": context.method,
            }
            try:
                result = await call_next(context)

                # BUG: The FastMCP type annotations seem to be incorrect.
                # This method typically returns fastmcp.tools.tool.ToolResult.
                if isinstance(result, mt.CallToolResult):
                    telemetry_data["tool_result_is_error"] = result.isError
                telemetry_data["tool_result_length"] = (
                    sum(
                        len(block.text)
                        for block in result.content
                        if isinstance(block, mt.TextContent)
                    ),
                )

                return result

            except Exception as e:
                telemetry_data["tool_call_error"] = e.__class__.__name__
                telemetry_data["tool_result_is_error"] = True
                raise
            finally:
                telemetry_data["duration_seconds"] = timer.elapsed_seconds()
                telemetry.telemetry_instance.ping(
                    "mcp-server-tool-call", telemetry_data
                )



import inspect
import time

import anyio
import fastmcp.tools.tool
import pytest

from mcp_server_datahub.mcp_server import async_background, mcp


@pytest.mark.anyio
async def test_async_background() -> None:
    @async_background
    def my_sleep(sec: float) -> None:
        time.sleep(sec)

    start_time = time.time()

    async with anyio.create_task_group() as tg:
        tg.start_soon(my_sleep, 0.5)
        tg.start_soon(my_sleep, 0.6)
        tg.start_soon(my_sleep, 0.7)

    end_time = time.time()
    duration = end_time - start_time
    # The calls should not be serialized, so the duration should be less than the sum of the durations.
    assert 0.5 <= duration < 1.8


def test_all_tools_are_async() -> None:
    # If any tools are sync, the tool execution will block the main event loop.
    for tool in mcp._tool_manager._tools.values():
        assert isinstance(tool, fastmcp.tools.tool.FunctionTool)
        assert inspect.iscoroutinefunction(tool.fn)
